#include <stdio.h>
#include <SocketHandler.h>
#include <TcpSocket.h>
#include <Mutex.h>
#include <StdoutLog.h>
#include <ListenSocket.h>
#include <Lock.h>


class InfoSocket : public TcpSocket
{
public:
	InfoSocket(ISocketHandler& h) : TcpSocket(h) {}
	~InfoSocket() {}

	void OnAccept() {
		Send("Welcome, " + GetRemoteAddress() + "\n");
	}
};


class MyHandler : public SocketHandler
{
public:
	MyHandler(Mutex m,StdoutLog *p) : SocketHandler(m, p) {}
	~MyHandler() {}

	void SendInfo(const std::string& msg) {
		for (socket_m::iterator it = m_sockets.begin(); it != m_sockets.end(); it++)
		{
			InfoSocket *p = dynamic_cast<InfoSocket *>((*it).second);
			if (p)
			{
				p -> Send(msg);
			}
		}
	}
};


class MySocket : public TcpSocket
{
public:
	MySocket(ISocketHandler& h) : TcpSocket(h) {
		SetLineProtocol();
	}
	~MySocket() {}

	void OnAccept() {
		Send("Welcome, " + GetRemoteAddress() + "\n");
#ifdef ENABLE_DETACH
		if (!Detach())
		{
			SetCloseAndDelete();
		}
#endif
	}

#ifdef ENABLE_DETACH
	void OnDetached() {
		Lock lck(MasterHandler().GetMutex());
printf("OnDetached()\n");
		static_cast<MyHandler&>(MasterHandler()).SendInfo(GetRemoteAddress() + ": New connection\n");
	}
#endif

	void OnLine(const std::string& line) {
		Lock lck(MasterHandler().GetMutex());
		static_cast<MyHandler&>(MasterHandler()).SendInfo(GetRemoteAddress() + ": " + line + "\n");
	}
};


int main(int argc,char *argv[])
{
	Mutex lock;
	StdoutLog log;
	MyHandler h(lock, &log);
	ListenSocket<MySocket> l(h);
	if (l.Bind(11001))
	{
		printf("Bind() failed\n");
		exit(-1);
	}
	h.Add(&l);
	ListenSocket<InfoSocket> l2(h);
	if (l2.Bind(11002))
	{
		printf("Bind() failed\n");
		exit(-2);
	}
	h.Add(&l2);
	bool quit = false;
	while (!quit)
	{
		h.Select(1, 0);
	}
}


